/**************************************************************************\
*   BC 1.0, WC 10.0, GCC 2.7                    Source: initvars.c
*   Host platform:   DOS                        Code:   ASCII-7
*   Target platform: DOS                        Code:   ASCII-7
*   Project name:    PGPEd
*   Last change:     14.11.98
*   Programmer:      Andrew Porokhnyak
****************************************************************************
*
\**************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>

#include "defines.h"
#include "config.h"
#include "initvars.h"
#include "getw.h"
#include "tools.h"


char fname[MAXPATH]="";                 /*file to edit*/
char ctname[MAXPATH];                   /*cipher text*/
char ptname[MAXPATH];                   /*plain text*/
char svname[MAXPATH];                   /*save PGP-copy*/
char ct_new=1;                          /*is ciphertext new?*/

char editbin[MAXPATH]=EDITBIN;          /*editor to invoke*/
char viewbin[MAXPATH]=VIEWBIN;
char pgpbin[MAXPATH]=PGPBIN;

char **envl;
char pgppath[MAXPATH]=PGPPATH;          /*PGPPATH env variable*/
char tzone[32]=TZONE;                   /*TZ ev variable*/

int pgprc_encr=PGPRC_ENCR;              /*PGP retc for OK encryption*/
int pgprc_decr=PGPRC_DECR;              /*PGP retc for OK decryption*/


/***************************************************************************
***************************************************************************/
typedef struct _KEYWORD KEYWORD;
struct _KEYWORD {
    const char *key;
    void (*callback) (KEYWORD *key, char *s);
    void *var;
    int option1;
    int option2;
};


static void read_int(KEYWORD *, char *);
static void read_string(KEYWORD *, char *);


#define DONT_CHECK 0x7fff

KEYWORD keywords[] = {
    {"editor", read_string, editbin, 0, MAXPATH},
    {"viewer", read_string, viewbin, 0, MAXPATH},
    {"pgp", read_string, pgpbin, 0, MAXPATH},
    {"pgppath", read_string, pgppath, 'd', MAXPATH},
    {"tz", read_string, tzone, 0, sizeof(tzone)},
    {"pgprc_encr", read_int, &pgprc_encr, 0, DONT_CHECK},
    {"pgprc_decr", read_int, &pgprc_decr, 0, DONT_CHECK},
    {NULL, NULL, NULL, 0, 0}
};

static char *path;
static int line;

/*
#define TEST(var) if (!*var) Log(0, "%s: "#var" should be defined", path)
*/

/*
 *
 */
void readcfg (char *_path) {
    FILE *in;
    char buf[MAXCFGLINE + 1];
    char *w;

    line=0;
    path=_path;

    if ((in = fopen(path, "rt")) == 0) {
        /* Log(1, "%s: %s", path, strerror(errno)); */
        return;
    }

    while (!feof (in)) {
        if (!fgets(buf, sizeof(buf), in))
            break;
        ++line;

        if ((w=getword(buf, 1)) != 0) {
            int j;

            for (j=0; keywords[j].key; ++j)
                if (!strcasecmp(keywords[j].key, w))
                    break;

            if (keywords[j].key) {
                keywords[j].callback(keywords + j, buf);
            } else {
                Log(1, "(%s:%i) '%s' - unknown keyword", path, line, w);
            }
            free (w);
        }
    }
    fclose (in);

} /*----------------------------------------------------------------------*/


/*
 * parsecl() -- parse command line, may set some global variables
 * return programm run mode
 */
int parsecl(const int argc, char **argv) {
    int i;
    int rmode=(argc ? RUN_EDIT : RUN_HELP);

    for (i=1; i<argc; i++) {
        if (!strcmp(argv[i], "-?") || !strcmp(argv[i], "-h") ||
            !strcmp(argv[i], "--help")) {
            return RUN_HELP;
        }
        if (!strcmp(argv[i], "--version")) {
            return RUN_VERSION;
        }
        if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--view")) {
            rmode=RUN_VIEW;
        }
        if (argv[i][0]!='-') {
            strcpy(fname, argv[i]);
        }
    }

    if (!fname[0]) rmode=RUN_HELP;

    return rmode;
} /*----------------------------------------------------------------------*/


/*
 * initvars()
 */
void initvars(void) {
    int h;
    int i;
    char s[256], *p;

    /* ptname <- fname without extention */
    strncpy(ptname, fname, MAXPATH);
    for (i=strlen(ptname); i>0; i--) {
        if (ptname[i]==PATH_DELIM[0]) break;
        if (ptname[i]=='.') {
            ptname[i]=(char) 0;
            break;
        }
    }
    /* ctname <- ptname + CT_EXT; svname <- ptname + SV_EXT */
    strncpy(ctname, ptname, MAXPATH);
    strncpy(svname, ptname, MAXPATH);
    strcat(ctname, CT_EXT);
    strcat(svname, SV_EXT);
    /* fname <- fname without extention (already done) and path */
    p=ptname;
    for (i=strlen(ptname); i>0; i--)
        if (ptname[i]==PATH_DELIM[0]) {
            p=ptname+i+1;
            break;
        }
    strncpy(fname, p, MAXPATH);

#ifdef HAVE_SPAWN
    /* Set some environment variables for spawned programs */
    if ((p=getenv("PGPPATH")) != NULL)
        strncpy(pgppath, p, strlen(p));
    if ((p=getenv("TZ")) != NULL)
        strncpy(tzone, p, strlen(p));
    i=0;
    envl=malloc((2+1)*sizeof(char *));
    if (pgppath) {
        sprintf(s, "PGPPATH=%s", pgppath);
        envl[i]=malloc(strlen(s)+1);
        strncpy(envl[i++], s, strlen(s)+1);
    }
    if (tzone) {
        sprintf(s, "TZ=%s", tzone);
        envl[i]=malloc(strlen(s)+1);
        strncpy(envl[i++], s, strlen(s)+1);
    }
    envl[i]=NULL;
#endif

    /* expand macro '$' to pgppath */
    if (pgpbin[0]=='$') {
        strncpy(s, (pgpbin+1), MAXPATH);
        strncpy(pgpbin, pgppath, MAXPATH);
        if (pgpbin[strlen(pgpbin)] != PATH_DELIM[0])
            strcat(pgpbin, PATH_DELIM);
        strcat(pgpbin, s);
    }

    /* check is ciphertext new */
    h=open(ctname, O_RDONLY);
    if (h!=-1) {
        ct_new=0;
        close(h);
    }


} /*----------------------------------------------------------------------*/

/*
 *  METHODS TO PROCESS KEYWORDS' ARGUMETS
 */

static void check_dir_path (char *s) {
    if (s) {
        char *w = s + strlen (s) - 1;

        while (w >= s && (*w == '/' || *w == '\\'))
            *(w--) = 0;
    }
}


/*
 * Read a string (key->option1 == 0)
 * or a directory name (key->option1 == 'd')
 * or a file name (key->option1 == 'f')
 */
static void read_string (KEYWORD *key, char *s) {

    struct stat sb;
    char *target = (char *) (key->var);
    char *w;

    if ((w = getword (s, 2)) != 0 && getword (s, 3) == 0) {
        strncpy (target, w, key->option1 == 0 ? key->option2 : MAXPATH);
        free (w);

        if (key->option1 != 0) {
            w = target + strlen (target) - 1;
            while (w >= target && (*w == '/' || *w == '\\')) {
                if (key->option1 == 'f') {
                    Log(1, "(%s:%i) unexpected `%c' at the end of filename",
                    path, line, *w); }
                *(w--) = 0;
            }
            if (key->option1 == 'd' && (stat(target, &sb) == -1 ||
                      !(sb.st_mode & S_IFDIR))) {
                Log(1, "(%s:%i) %s: incorrect directory\n", path, line,
                target);
            }
        }
    } else
        Log(1, "(%s:%i) the syntax is incorrect\n", path, line);

} /*----------------------------------------------------------------------*/


static void read_int (KEYWORD *key, char *s) {

    int *target = (int *) (key->var);
    char *w;

    if ((w = getword (s, 2)) != 0 && getword (s, 3) == 0) {
        *target = atoi (w);
        if ((key->option1 != DONT_CHECK &&
                *target < key->option1) ||
            (key->option2 != DONT_CHECK &&
                *target > key->option2)) {
            Log(1, "(%s:%i) %i: incorrect value\n", path, line,
            *target);
        }
        free (w);
    } else
        Log(1, "(%s:%i) the syntax is incorrect\n", path, line);

} /*----------------------------------------------------------------------*/

/*** EOF initvars.c *******************************************************/